require_relative 'connection'
require_relative 'request'
require_relative 'venue'

module TicketingHub
  class Client
    include TicketingHub::Connection
    include TicketingHub::Request

    attr_accessor *Configuration::VALID_OPTIONS_KEYS

    def self.from_oauth_password(email, password)
      token = TicketingHub.post 'oauth/token', { grant_type: 'password', username: email, password: password,
        client_id: TicketingHub.client_id, client_secret: TicketingHub.client_secret }
      new(access_token: token.access_token, refresh_token: token.refresh_token, expires_at: Time.now + token.expires_in)
    end

    def initialize(options={})
      options = TicketingHub.options.merge options
      Configuration::VALID_OPTIONS_KEYS.each do |key|
        send "#{key}=", options[key]
      end
    end

    def venues
      get 'venues', {}, TicketingHub::Venue
    end

    def user
      get('user').first
    end
  end
end